@@ -2,8 +2,7 @@
  controller.js -- The GBrowse controller object
 
  Lincoln Stein <lincoln.stein@gmail.com>
- Ben Faga <ben.faga@gmail.com>
- $Id: controller.js,v 1.88 2009/03/27 02:10:09 lstein Exp $
+ $Id: controller.js,v 1.68 2008/12/28 05:08:34 lstein Exp $
 
 Indentation courtesy of Emacs javascript-mode 
 (http://mihai.bazon.net/projects/emacs-javascript-mode/javascript.el)
@@ -30,7 +29,6 @@ var detail_container_id     = 'detail_panels';
 var external_utility_div_id = 'external_utility_div'; 
 var edit_upload_form_id     = 'edit_upload_form';
 var page_title_id           = 'page_title';
-var galaxy_form_id          = 'galaxy_form';
 var visible_span_id         = 'span';
 var search_form_objects_id  = 'search_form_objects';
 
@@ -101,15 +99,6 @@ var GBrowseController = Class.create({
     }
   }, // end unregister_track
 
-  unregister_gbtrack:
-  function (gbtrack) {
-      var id_hash = this.gbtrackname_to_id.get(gbtrack.track_name);
-      if (id_hash != null)
-           id_hash.unset(gbtrack.track_id);
-      if (this.gbtracks.get(gbtrack.track_id) != null)
-          this.gbtracks.unset(gbtrack.track_id);
-  },
-
   // Pass an iterator to execute something on each track
   // Call as this.each_track(function(track){}) to iterate over all gbtracks.
   // Call as this.each_track('track_name',function(track){}) to iterate over
@@ -147,8 +136,7 @@ var GBrowseController = Class.create({
   // Sets the time key for the tracks so we know if one is outdated
   set_last_update_keys:
   function (track_keys) {
-    var last_update_key  = create_time_key();
-    this.last_update_key = last_update_key;
+    this.last_update_key = create_time_key();
 
     var track_key_hash = new Hash;
     for (var track_name in track_keys)
@@ -156,7 +144,7 @@ var GBrowseController = Class.create({
 
     this.each_track(function(gbtrack) {
         if (track_key_hash.get(gbtrack.track_name) != null) {
-	   gbtrack.set_last_update_key(last_update_key);
+	   gbtrack.set_last_update_key(this.last_update_key);
 	}
     });
   }, // end set_last_update_keys
@@ -198,7 +186,7 @@ var GBrowseController = Class.create({
         cursor:     'text'
     });
     image.setOpacity(1);
-    image.ancestors()[0].setStyle({width: bar_obj.width+'px'});
+    image.parentNode.setStyle({width: bar_obj.width+'px'});
   },
 
   append_child_from_html:
@@ -238,8 +226,7 @@ var GBrowseController = Class.create({
     for (var i = 0; i < section_names.length; i++) {
       request_str += "&section_names="+section_names[i];
     }
-
-    new Ajax.Request(document.URL,{
+    new Ajax.Request('#',{
       method:     'post',
       parameters: request_str,
       onSuccess: function(transport) {
@@ -249,11 +236,7 @@ var GBrowseController = Class.create({
           html    = section_html[section_name];
           $(section_name).innerHTML = html;
 	  if (scroll_there)
-	    new Effect.ScrollTo(section_name);
-	    if ((section_name=="search_form_objects") 
-	      && ($('autocomplete_choices') != null)) {
-	    	initAutocomplete();
-	    }
+	    $(section_name).scrollTo();
         }
       }
     });
@@ -265,13 +248,13 @@ var GBrowseController = Class.create({
 
      var param = {set_display_option: 1};
      param[option] = value;
-     new Ajax.Request(document.URL,
+     new Ajax.Request('#',
             {
 		    method: 'post', 
 		    parameters: param,
-		    onComplete:  function (transport) {
-		      Controller.update_coordinates('left 0'); // causes an elegant panel refresh
-		    } 
+		    onSuccess:  function (transport) { 
+		    Controller.update_coordinates('left 0'); // causes an elegant panel refresh
+		} 
             }
      );
 
@@ -279,26 +262,19 @@ var GBrowseController = Class.create({
 
   // Signal Change to Server Methods ********************************
   set_track_visibility:
-  function(track_id,visible) {
+  function(track_name,visible) {
 
-    var gbtrack  = this.gbtracks.get(track_id);
-    if (gbtrack == null) return;
+    this.each_track(track_name,function(gbtrack) {
 
-    var track_name = gbtrack.track_name;
-
-    this.each_track(track_id,function(gbtrack) {
-
-      new Ajax.Request(document.URL,{
+      new Ajax.Request('#',{
         method:     'post',
         parameters: {
           set_track_visibility:  1,
-          visible:             visible,
-          track_name:          track_name
+          visible:               visible,
+          track_name:            track_name
         },
         onSuccess: function(transport) {
-          if (visible && 
-	      gbtrack.get_last_update_key() == null ||
-	      gbtrack.get_last_update_key() < Controller.last_update_key) {
+          if (visible && gbtrack.get_last_update_key() < Controller.last_update_key) {
             Controller.rerender_track(gbtrack.track_id);
           }
         }
@@ -311,11 +287,11 @@ var GBrowseController = Class.create({
   first_render:
   function()  {
 
-    new Ajax.Request(document.URL,{
+    new Ajax.Request('#',{
       method:     'post',
       parameters: {first_render: 1},
       onSuccess: function(transport) {
-	var results             = transport.responseJSON;
+        var results             = transport.responseJSON;
         var track_keys          = results.track_keys;
         Controller.segment_info = results.segment_info;
         $('details_msg').innerHTML = results.details_msg;
@@ -341,30 +317,30 @@ var GBrowseController = Class.create({
     this.each_track(function(gbtrack) {
          if ($(gbtrack.track_image_id) != null)
 	    $(gbtrack.track_image_id).setOpacity(0.3);
-	 //  else
-	 //  alert('REPORT THIS BUG: element '+gbtrack.track_image_id+' should not be null');
+         else
+            alert('REPORT THIS BUG: element '+gbtrack.track_image_id+' should not be null');
     });
     
-    new Ajax.Request(document.URL,{
+    new Ajax.Request('#',{
       method:     'post',
       parameters: {navigate: action},
       onSuccess: function(transport) {
-	var results                 = transport.responseJSON;
-        Controller.segment_info     = results.segment_info;
-	var track_keys              = results.track_keys;
-	var overview_scale_bar_hash = results.overview_scale_bar;
-	var region_scale_bar_hash   = results.region_scale_bar;
-	var detail_scale_bar_hash   = results.detail_scale_bar;
+	var results                     = transport.responseJSON;
+        Controller.segment_info         = results.segment_info;
+	    var track_keys              = results.track_keys;
+	    var overview_scale_bar_hash = results.overview_scale_bar;
+	    var region_scale_bar_hash   = results.region_scale_bar;
+	    var detail_scale_bar_hash   = results.detail_scale_bar;
         Controller.set_last_update_keys(track_keys);
-
+    
         if (overview_scale_bar_hash){
-	    Controller.update_scale_bar(overview_scale_bar_hash);
+          Controller.update_scale_bar(overview_scale_bar_hash);
         }
         if (region_scale_bar_hash){
-	    Controller.update_scale_bar(region_scale_bar_hash);
+          Controller.update_scale_bar(region_scale_bar_hash);
         }
         if (detail_scale_bar_hash){
-	    Controller.update_scale_bar(detail_scale_bar_hash);
+          Controller.update_scale_bar(detail_scale_bar_hash);
         }
     
         // Update the segment sections
@@ -381,28 +357,20 @@ var GBrowseController = Class.create({
 
   }, // end update_coordinates
 
-  scroll:
-  function (direction,length_units) {
-     var length = this.segment_info.detail_stop - this.segment_info.detail_start + 1;
-     this.update_coordinates(direction + ' ' + Math.round(length_units*length));
-  }, // end scroll
-
   add_track:
-  function(track_name, onSuccessFunc, force) {
+  function(track_name, onSuccessFunc) {
     var track_names = new Array(track_name);
-    this.add_tracks(track_names,onSuccessFunc,force);
+    this.add_tracks(track_names, onSuccessFunc);
   },
 
   add_tracks:
-  function(track_names, onSuccessFunc, force) {
-
-    if (force == null) force=false;
+  function(track_names, onSuccessFunc) {
 
     var request_str = "add_tracks=1";
     var found_track = false;
     for (var i = 0; i < track_names.length; i++) {
       var track_name = track_names[i];
-      if ( force || !this.track_exists(track_name) ) {
+      if ( !this.track_exists(track_name) ) {
         request_str += "&track_names="+encodeURIComponent(track_name);
         found_track = true;
       }
@@ -410,7 +378,7 @@ var GBrowseController = Class.create({
 
     if (!found_track) return false;
 
-    new Ajax.Request(document.URL,{
+    new Ajax.Request('#',{
       method:     'post',
       parameters: request_str,
       onSuccess: function(transport) {
@@ -423,13 +391,8 @@ var GBrowseController = Class.create({
         var get_tracks = false;
 
         for (var ret_track_id in track_data){
-
-	  if (Controller.gbtracks.get(ret_track_id) != null) {
-	     continue; //oops already know this one
-	  }
-
           var this_track_data = track_data[ret_track_id];
-          var ret_gbtrack     = Controller.register_track(
+          var ret_gbtrack = Controller.register_track(
 	    ret_track_id,
             this_track_data.track_name,
             'standard',
@@ -469,7 +432,7 @@ var GBrowseController = Class.create({
        $(gbtrack.track_image_id).setOpacity(0.3);
        Controller.set_last_update_key(gbtrack);
 
-       new Ajax.Request(document.URL,{
+       new Ajax.Request('#',{
          method:     'post',
          parameters: {
            rerender_track:  1,
@@ -490,7 +453,7 @@ var GBrowseController = Class.create({
              Controller.get_remaining_tracks(track_keys,1000,1.1,time_key);
            } 
    	   if (scroll_there) {
-	      new Effect.ScrollTo(gbtrack.track_div_id);
+	      $(gbtrack.track_div_id).scrollTo();
 	   }
          } // end onSuccess
        }); // end Ajax.Request
@@ -536,7 +499,7 @@ var GBrowseController = Class.create({
 
     if (finished) return;
 
-    new Ajax.Request(document.URL,{
+    new Ajax.Request('#',{
       method:     'post',
       parameters: $H({ retrieve_multiple: 1, 
                        track_ids:     track_ids
@@ -595,25 +558,16 @@ var GBrowseController = Class.create({
   // Track Configure Methods ****************************************
 
   reconfigure_track:
-  function(track_id, form_element) {
-
-    if (form_element==null)
-       form_element = $("track_config_form");
-    else
-       Element.extend(form_element);
-
-    var show_box   = form_element['show_track'];
-    var show_track = $(show_box).getValue();
-
-    new Ajax.Request(document.URL,{
+  function(track_id, serialized_form, show_track) {
+    new Ajax.Request('#',{
       method:     'post',
-      parameters: form_element.serialize() +"&"+ $H({
+      parameters: serialized_form +"&"+ $H({
             reconfigure_track: track_id
           }).toQueryString(),
       onSuccess: function(transport) {
         var track_div_id = Controller.gbtracks.get(track_id).track_div_id;
         Balloon.prototype.hideTooltip(1);
-        if (show_track == track_id){
+        if (show_track){
           Controller.rerender_track(track_id,true);
         }
         else{
@@ -634,23 +588,17 @@ var GBrowseController = Class.create({
   function(div_id) {
     var plugin_base  = document.pluginform.plugin.value;
     this.update_sections(new Array(div_id), '&plugin_base='+plugin_base);
-    new Effect.ScrollTo(div_id);
   },
 
   reconfigure_plugin:
-  function(plugin_action,plugin_track_id,pc_div_id,plugin_type,form_element) {
-    if (form_element==null)
-       form_element = $("configure_plugin");
-    else
-       Element.extend(form_element);
-
-    new Ajax.Request(document.URL,{
+  function(plugin_action,plugin_track_id,pc_div_id,plugin_type) {
+    var form_element = $("configure_plugin");
+    new Ajax.Request('#',{
       method:     'post',
       parameters: form_element.serialize() +"&"+ $H({
             plugin_action: plugin_action,
             reconfigure_plugin: 1
           }).toQueryString(),
-
       onSuccess: function(transport) {
         Controller.wipe_div(pc_div_id); 
 
@@ -679,10 +627,10 @@ var GBrowseController = Class.create({
       var loc_str = "?plugin="+plugin_base+";plugin_action="+encodeURI(plugin_action);
       if(source == 'config'){
         var form_element = $("configure_plugin");
-        window.open(loc_str + ";" + form_element.serialize());
+        window.location=loc_str + ";" + form_element.serialize();
       }
       else{
-	window.open(loc_str);
+        window.location=loc_str;
       }
     }
     else if (plugin_type == 'filter'){
@@ -705,7 +653,6 @@ var GBrowseController = Class.create({
   function(edit_file) {
     var gbtrack  = this.gbtracks.get(decodeURIComponent(edit_file));
     var basename = gbtrack!=null ? gbtrack.track_name : edit_file;
-    visibility('upload_tracks_panel',1);
     Controller.update_sections(new Array(external_utility_div_id), 
    	    '&edit_file='+basename,true);
   },
@@ -713,7 +660,7 @@ var GBrowseController = Class.create({
   commit_file_edit:
   function(edited_file) {
     var form_element = $(edit_upload_form_id);
-    new Ajax.Request(document.URL,{
+    new Ajax.Request('#',{
       method:     'post',
       parameters: form_element.serialize() +"&"+ $H({
             edited_file: edited_file,
@@ -722,47 +669,20 @@ var GBrowseController = Class.create({
       onSuccess: function(transport) {
         var results      = transport.responseJSON;
         var file_created = results.file_created;
-	var tracks       = results.tracks;
-
         Controller.wipe_div(external_utility_div_id); 
 
-	var current_tracks = new Hash();
-	Controller.each_track(edited_file,function(gbtrack){
-           current_tracks.set(gbtrack.track_id,gbtrack);
-	});
-
-	var new_tracks = new Hash();
-	tracks.each(function(trackid){
-           new_tracks.set(trackid,1);
-	});
-
-	var tracks_to_delete = current_tracks.keys().findAll(function(trackid) {
-            return new_tracks.get(trackid)==null;
-	});
-	var tracks_to_add    = new_tracks.keys().findAll(function(trackid) {
-	    return current_tracks.get(trackid)==null;
-	});
-	var tracks_to_update = new_tracks.keys().findAll(function(trackid) {
-	    return current_tracks.get(trackid)!= null;
-	});
-
-	tracks_to_update.each(function(id) { 
-	   Controller.rerender_track(id,true);
-	   Controller.update_sections(new Array(external_listing_id));
-	});
-
-	tracks_to_delete.each(function(id) { 
-	   var gbtrack = current_tracks.get(id);
-	   actually_remove(gbtrack.track_div_id);
-	   Controller.unregister_gbtrack(gbtrack);
-	   Controller.update_sections(new Array(track_listing_id,external_listing_id));
-	});
-
-	if (tracks_to_add.length > 0)
+        if ( 1 == file_created ){
           Controller.add_track(edited_file, function(){
             Controller.update_sections(new Array(track_listing_id,external_listing_id));
-        },true);	
-
+          })
+        }
+        else{
+        // update track if it exists
+	 Controller.each_track(edited_file,function(gbtrack) {
+             Controller.rerender_track(gbtrack.track_id,true);
+         });
+         Controller.update_sections(new Array(external_listing_id));
+        }
       } // end onSuccess
     });
   },
@@ -770,9 +690,7 @@ var GBrowseController = Class.create({
   delete_upload_file:
   function(file_name) {
 
-    $(external_listing_id).innerHTML='<p><b style="background-color:yellow">Working...</b></p>';
-
-    new Ajax.Request(document.URL,{
+    new Ajax.Request('#',{
       method:     'post',
       parameters: {
         delete_upload_file: 1,
@@ -794,9 +712,7 @@ var GBrowseController = Class.create({
   function(eurl) {
     if ( eurl == '') return;
 
-    $(external_listing_id).innerHTML='<p><b style="background-color:yellow">Working...</b></p>';
-
-    new Ajax.Request(document.URL,{
+    new Ajax.Request('#',{
       method:     'post',
       parameters:{
             add_url:    1,
@@ -815,53 +731,7 @@ var GBrowseController = Class.create({
           });
         }
     });
-  },
-
-  // Utility methods *********************************
-  show_error:
-  function (message,details) {
-      var outerdiv    = $('errordiv');
-      var innerdiv    = $('errormsg');
-      var detailsdiv  = $('errordetails');
-      if (innerdiv != null) {
-          var caption = detailsdiv.visible() ? 'Hide details' : 'Show details';
-	  innerdiv.innerHTML = message +
-                               ' <a id="detailscaption" href="javascript:void(0)" onClick="Controller.show_hide_errordetails()">'
-			       +caption
-			       +'</a>';
-      }			     
-      if (detailsdiv != null) {
-          detailsdiv.innerHTML  = details;
-      }
-      if (outerdiv != null) {
-         scroll(0,0);
-	 new Effect.BlindDown(outerdiv);
-      }
-  },
-
-  hide_error:
-  function () {
-      var outerdiv   = $('errordiv');
-      var detailsdiv = $('errordetails');
-      if (outerdiv != null)
-	  new Effect.BlindUp(outerdiv);
-      return false;
-  },
-
- show_hide_errordetails:
- function () {
-    var detailsdiv = $('errordetails');
-    var caption    = $('detailscaption');
-    if (detailsdiv == null) return;
-    if (caption    == null) return;
-    if (detailsdiv.visible()) {
-       caption.innerHTML="Show details";
-       detailsdiv.hide();
-    } else {
-       caption.innerHTML="Hide details";
-       detailsdiv.show();
-    }
- }
+  }
 
 });
 
@@ -869,21 +739,13 @@ var Controller = new GBrowseController; // singleton
 
 function initialize_page() {
   //event handlers
-    [page_title_id,visible_span_id,galaxy_form_id,search_form_objects_id].each(function(el) {
+  [page_title_id,visible_span_id,search_form_objects_id].each(function(el) {
     if ($(el) != null) {
       Controller.segment_observers.set(el,1);
     }
   });
   
   Controller.first_render();
-
-  // The next statement is to avoid the scalebars from being "out of sync"
-  // when manually advancing the browser with its forward/backward buttons.
-  // Unfortunately it causes an infinite loop when there are multiple regions!
-  if ($(detail_container_id) != null)
-      Controller.update_coordinates('left 0');
-
-  // These statements get the rubberbanding running.
   Overview.prototype.initialize();
   Region.prototype.initialize();
   Details.prototype.initialize();
@@ -891,11 +753,8 @@ function initialize_page() {
 
 // set the colors for the rubberband regions
 function set_dragcolors(color) {
-    if (overviewObject != null)
      overviewObject.background = color;
-    if (regionObject != null)
      regionObject.background   = color;
-    if (detailsObject != null)
      detailsObject.background  = color;
 }
 
